# -*- coding: utf-8 -*-

"""
	Usage:
		search_google [q] [--optional]
		search_google [-positional] ...
	
	A command line tool for Google web and image search.
	
	Positional arguments:
		q									keyword query
		-h								show this help message and exit
		-i								show documentation in browser
		-a								show optional arguments in browser
		-s <arg>=<value>	set default optional arguments
		-r <arg>					remove default arguments
		-v								view default arguments
		-d								reset default arguments
	
	Optional arguments:
		--num							num of results (default: 3)
		--searchType			'image' or unassigned for web search
		--dateRestrict		time period of search
		--start						index of first result
		--fileType				format for image search (default: png)
		--save_links			path for text file of links
		--save_metadata		path for metadata JSON file
		--save_downloads	path for directory of link downloads
		--option_silent		'True' to disable preview
		--option_preview	num of results to preview
		
		For more arguments use: search_google -a
		
	Examples:
		
		Set developer and search engine key arguments
			> search_google -s build_developerKey="dev_key"
			> search_google -s cx="cse_key"
		
		Web search for keyword "cat"
			> search_google cat
		
		Search for "cat" images
			> search_google cat --searchType=image
		
		Download links to directory
			> search_google cat --save_downloads=downloads
	
	For more information visit use: search_google -i
"""

from os.path import isfile
from pkg_resources import resource_filename, Requirement
from pprint import pprint
from sys import argv
from webbrowser import open_new_tab

import json
import kwconfig
import search_google.api

_doc_link = 'https://github.com/rrwen/search_google'
_cse_link = 'https://developers.google.com/resources/api-libraries/documentation/customsearch/v1/python/latest/customsearch_v1.cse.html'
		
def run(argv=argv):
	"""Runs the search_google command line tool.
	
	This function runs the search_google command line tool 
	in a terminal. It was intended for use inside a py file 
	(.py) to be executed using python.
	
	Notes:
		* ``[q]`` reflects key ``q`` in the ``cseargs`` parameter for :class:`api.results`
		* Optional arguments with ``build_`` are keys in the ``buildargs`` parameter for :class:`api.results`
	
		For distribution, this function must be defined in the following files::
			
			# In 'search_google/search_google/__main__.py'
			from .cli import run
			run()
			
			# In 'search_google/search_google.py'
			from search_google.cli import run
			if __name__ == '__main__':
				run()
			
			# In 'search_google/__init__.py'
			__entry_points__ = {'console_scripts': ['search_google=search_google.cli:run']}
	
	Examples::
		
		# Import google_streetview for the cli module
		import search_google.cli
		
		# Create command line arguments
		argv = [
			'cli.py',
			'google',
			'--searchType=image',
			'--build_developerKey=your_dev_key',
			'--cx=your_cx_id'
			'--num=1'
		]
		
		# Run command line
		search_google.cli.run(argv)
		
	"""
	config_file = kwconfig.manage(
		file_path=resource_filename(Requirement.parse('search_google'), 'search_google/config.json'),
		defaults={
			'build_serviceName': 'customsearch',
			'build_version': 'v1',
			'num': 3,
			'option_silent': 'False',
			'option_preview' : 10})
	
	# (commands) Main command calls
	if len(argv) > 1:
		if argv[1] == '-i': # browse docs
			open_new_tab(_doc_link)
			exit()
		elif argv[1] == '-a': # browse arguments
			open_new_tab(_cse_link)
			exit()
	config_file.command(argv, i=1, doc=__doc__, quit=True, silent=False)
	
	# (parse_args) Parse command arguments into dict
	kwargs = kwconfig.parse(argv[2:])
	kwargs['q'] = argv[1]
	kwargs = config_file.add(kwargs)
	
	# (split_args) Split args into build, cse, and save arguments
	buildargs = {}
	cseargs = {}
	saveargs = {}
	optionargs = {}
	for k, v in kwargs.items():
		if 'build_' == k[0:6]:
			buildargs[k[6:]] = v
		elif 'save_' == k[0:5]:
			saveargs[k[5:]] = v
		elif 'option_' == k[0:7]:
			optionargs[k[7:]] = v
		else:
			cseargs[k] = v
	
	# (cse_results) Get google api results
	results = search_google.api.results(buildargs, cseargs)
	
	# (cse_print) Print a preview of results
	if optionargs['silent'].lower() != 'true':
		results.preview(n=optionargs['preview'])
	
	# (cse_save) Save links and metadata
	if 'links' in saveargs:
		results.save_links(saveargs['links'])
	if 'metadata' in saveargs:
		results.save_metadata(saveargs['metadata'])
	
	# (cse_download) Download links
	if 'downloads' in saveargs:
		results.download_links(saveargs['downloads'])
		


# -*- coding: utf-8 -*-

from apiclient.discovery import build
from os import makedirs, path
from urllib.request import urlopen

import json
import shutil
import requests

class results:
	"""Google Custom Search Engine (CSE) API results.
	
	Uses the `Google Custom Search Engine API <https://developers.google.com/api-client-library/python/apis/customsearch/v1>`_ 
	to search webpages and images using queries.
	
	Args:
		buildargs (dict):
			Named arguments for `googleapiclient.build <https://google.github.io/google-api-python-client/docs/epy/googleapiclient.discovery-module.html#build>`_.
		cseargs (dict):
			Named arguments for `cse.list <https://developers.google.com/resources/api-libraries/documentation/customsearch/v1/python/latest/customsearch_v1.cse.html>`_.
	
	Attributes:
		metadata (dict):
			object returned from `cse.list <https://developers.google.com/resources/api-libraries/documentation/customsearch/v1/python/latest/customsearch_v1.cse.html>`_.
		buildargs (dict):
			Same as argument ``buildargs`` for reference of inputs.
		csedargs (dict):
			Same as argument ``cseargs`` for reference of inputs.
	
	Examples: 
		::
		
			# Import the api module for the results class
			import search_google.api
			
			# Define buildargs for api api
			buildargs = {
				"serviceName": "customsearch",
				"version": "v1",
				"developerKey": "your_api_key"
			}
			
			# Define cseargs for search
			cseargs = {
				"q": "keyword query",
				"cx": "your_cse_id",
				"num": 3
			}
			
			# Create a results object
			results = search_google.api.results(buildargs, cseargs)
			
			# Preview the search results
			results.preview()
			
			# Obtain the url links from the search
			# Links are inside results['items'] list
			links = results.get_values('items', 'link')
			
			# Obtain the url links from the search
			links = results.links
			
			# Save the search result metadata to a json file
			results.save_metadata('metadata.json')
			
			# Save the search result links to a text file
			results.save_links('links.txt')
			
			# Download the search results to a directory
			results.download_links('downloads')
	"""
	def __init__(
		self,
		buildargs = {
			'serviceName': 'customsearch',
			'version': 'v1',
		},
		cseargs={
			'num': 3,
		}):
		self.buildargs = buildargs
		self.cseargs = cseargs
		self.metadata = build(**buildargs).cse().list(**cseargs).execute()
	
	def download_links(self, dir_path):
		"""Download web pages or images from search result links.
		
		Args:
			dir_path (str):
				Path of directory to save downloads of :class:`api.results`.links
		"""
		links = self.links
		if not path.exists(dir_path):
			makedirs(dir_path)
		for i, url in enumerate(links):
			ext = path.splitext(url)[1]
			ext = '.html' if ext == '' else ext
			file_name = self.cseargs['q'].replace(' ', '_') + '_' + str(i) + ext
			file_path = path.join(dir_path, file_name)
			r = requests.get(url, stream=True)
			if r.status_code == 200:
				with open(file_path, 'wb') as f:
					r.raw.decode_content = True
					shutil.copyfileobj(r.raw, f)
		
	def get_values(self, k, v):
		"""Get a list of values from the key value metadata attribute.
		
		Args:
			k (str):
				Key in :class:`api.results`.metadata
			v (str):
				Values from each item in the key of :class:`api.results`.metadata
		
		Returns:
			A list containing all the ``v`` values in the ``k`` key for the	 :class:`api.results`.metadata attribute.
		"""
		metadata = self.metadata
		values = []
		if metadata != None:
			if k in metadata:
				for metav in metadata[k]:
					if v in metav:
						values.append(metav[v])
		return values
		
	def preview(self, n=10, k='items', kheader='displayLink', klink='link', kdescription='snippet'):
		"""Print a preview of the search results.
		
		Args:
			n (int):
				Maximum number of search results to preview
			k (str):
				Key in :class:`api.results`.metadata to represent
			kheader (str):
				Key in :class:`api.results`.metadata[``k``] to use as the header
			klink (str):
				Key in :class:`api.results`.metadata[``k``] to use as the link if image search
			kdescription (str):
				Key in :class:`api.results`.metadata[``k``] to use as the description
		"""
		if 'searchType' in self.cseargs:
			searchType = self.cseargs['searchType']
		else:
			searchType = None
		items = self.metadata[k]
	
		# (cse_print) Print results
		for i, kv in enumerate(items[:n]):
			
			# (print_header) Print result header
			header = '\n[' + str(i) + '] ' + kv[kheader]
			print(header)
			print('=' * len(header))
			
			# (print_image) Print result image file
			if searchType == 'image':
				link = '\n' + path.basename(kv[klink])
				print(link)
				
			# (print_description) Print result snippet
			description = '\n' + kv[kdescription]
			print(description)
	
	def save_links(self, file_path):
		"""Saves a text file of the search result links.
		
		Saves a text file of the search result links, where each link 
		is saved in a new line. An example is provided below::
			
			http://www.google.ca
			http://www.gmail.com
		
		Args:
			file_path (str):
				Path to the text file to save links to.
		"""
		data = '\n'.join(self.links)
		with open(file_path, 'w') as out_file:
			out_file.write(data)
		
	def save_metadata(self, file_path):
		"""Saves a json file of the search result metadata.
		
		Saves a json file of the search result metadata from :class:`api.results`.metadata.
		
		Args:
			file_path (str):
				Path to the json file to save metadata to.
		"""
		data = self.metadata
		with open(file_path, 'w')	 as out_file:
			json.dump(data, out_file)
		
	@property
	def links(self):
		"""list of str: Web links to search results using :func:`api.results.get_values`."""
		return self.get_values('items', 'link')
		


# -*- coding: utf-8 -*-

__name__ = 'search_google'
__author__ = 'Richard Wen'
__email__ = 'rrwen.dev@gmail.com'
__version__ = '1.0.9'
__license__ = 'MIT'
__description__ = 'A command line tool and module for Google API web and image search.'
__keywords__ = [
  'google',
  'api',
  'custom',
  'web',
  'image',
  'search',
  'engine',
  'cli',
  'cse',
  'command', 
  'line',
  'interface',
  'tool',
  'module']
__url__ = 'https://github.com/rrwen/search_google'
__download_url__ = 'https://github.com/rrwen/search_google/archive/master.zip'
__install_requires__ = [
  'google-api-python-client',
  'kwconfig',
  'requests'
]
__packages__ = ['search_google']
__package_data__ = {'search_google': ['config.json']}
__entry_points__ = {'console_scripts': ['search_google=search_google.cli:run']}
